/*** FILEHEADER ****************************************************************
 *
 *   FILENAME:    demo.c
 *   DATE:        04.08.2004
 *   AUTHOR:       (c) Christian Stadler
 *
 * DESCRIPTION: RFM12 wireless FSK transceiver module demo program.
 *              To be used with PIC18F4XX demo board.
 *              RFM12 module connection to PIC18F4XX demo board:
 *                  RD7 = SCK
 *                  RD6 = SDI
 *                  RD5 = nSel
 *                  RD4 = SDO
 *                  RD3 = IRQ
 *
 ******************************************************************************/

/*** HISTORY OF CHANGE *********************************************************
 *
 *   $Log: /pic/rfm12demo/rfm12demo.c $
 * 
 * 8     1.11.10 13:42 Stadler
 * - make code compilable with C18 compiler and CCS compiler
 * 
 * 7     1.11.10 12:17 Stadler
 * - moved RFM12 pin configuration into rfm12conf.h
 * 
 * 6     31.10.10 17:27 Stadler
 * - separeated RFM12 driver code into separate file
 * 
 * 5     31.10.10 14:16 Stadler
 * - transmit 2 dummy bytes at the end of data transfer
 * 
 * 4     30.10.10 22:18 Stadler
 * - fixed RFM12_Init
 * - changed LED indication: Receiver keeps LED1 on
 * 
 * 3     30.10.10 21:39 Stadler
 * - increased init delay
 * - removed warnings
 * 
 * 2     30.10.10 20:23 Stadler
 * - updated code documentation
 * 
 * 1     30.10.10 18:49 Stadler
 * - RFM12 wireless FSK transceiver module demo
 *
 ******************************************************************************/

/*
=== STATUS LED ===
	1. nSEL
	2. SCK
	3. SDI (Input for the RFM12 - for our controller it is the output)
	4. nFFS
*/

// 4MHz Crystal, (XT oscillator)
#pragma config OSC 		=	XT 	

/* make compiler case sensitive */
#pragma case

/* To use the demo, 1 receiver and 1 transmitter is required. */
/* To generate HEX file for the receiver uncomment the #define MODE_RECEIVER */
/* and comment the #define MODE_TRANSMITTER. */
/* To generate HEX file for the receiver uncomment the #define */
/* MODE_TRANSMITTER and comment the #define MODE_RECEIVER. */

#define MODE_RECEIVER
//#define MODE_TRANSMITTER

#include <p18f4520.h>
#include "delay.h"
#include "led.h"
#include "defines.h"
#include "interrupt.h"
#include "rfm12conf.h"
/* RFM12 driver */
#include "rfm12.h"

/*******************************************************************************
 * FUNCTION:   void init(void)
 * PURPOSE:    Initializes PIC and external HW.
 ******************************************************************************/
void init(void)
{
	//No need for Analog Input for first time
	ADCON1 = 0x07;	//ADFM ADCS2   PCFG3 PCFG2 PCFG1 PCFG0
	//Set the register
	TRISD=0;
	TRISC=0;
	//button as input
	TRISAbits.TRISA0=1;
	//LED for showing RxTx success
	TRISAbits.TRISA1=0;
	//LED for showing Rx proccess
	TRISAbits.TRISA2=0;
	//Enable timer0 interrupt
	SetUpISRTimer0();
    /* switch off LEDs */
    set_led(false);
}



/*******************************************************************************
 * FUNCTION:   void receive(void)
 * PURPOSE:    Receive data.
 ******************************************************************************/
void receive(void)
{
    static uint16 rxcount = 0;
    uint8 rxbuf[40];
	int i = 0;
    
    /* init RX buffer */
    memset(rxbuf, 0, sizeof(rxbuf));
    iStatus = RFM12_Write(0x0000);
    RFM12_RxData(rxbuf, 32);
	for (i = 0; i < 40; i++)
	{
    	if (rxbuf[i] != 0)
		{
			//we received something!
			GUI_LED_TxRx = true;
		}
	}
	iStatus = RFM12_Write(0x0000);
	delay_ms(500);
	GUI_LED_TxRx = false;
}



/*******************************************************************************
 * FUNCTION:   void transmit(void)
 * PURPOSE:    Transmit data.
 ******************************************************************************/
void transmit(void)
{
    /*             0         1         2         3  */
    /*             01234567890123456789012345678901 */
    //uint8 txbuf[]="*** Hello World! RFM12 Demo! ***";
	uint8 txbuf[]="01234567";
	iStatus = RFM12_Write(0x0000);
    RFM12_TxData(txbuf, 8);
	iStatus = RFM12_Write(0x0000);
	if (iStatus & 0x8000)
	{
		//ready to send new data
		GUI_LED_TxRx = true;
	}
	else
	{
		GUI_LED_TxRx = false;
	}
	delay_ms(500);
	GUI_LED_TxRx = !RFM12_GetIRQ();
	delay_ms(500);
}


/*******************************************************************************
 * FUNCTION:   void main(void)
 * PURPOSE:    Main.
 ******************************************************************************/
void main(void)
{
    init();
    
    /* init RFM12 module */
    RFM12_Init();
	delay_us(100);
    RFM12_SetFreq(RFM12FREQ(433.92)); /* Rx/Tx frequency 433.92MHz */
    delay_us(100);
	RFM12_SetBandwidth(4, 1, 4);      /* 200kHz Bandwidth, -6dB LNA gain, DRSSI threshold: -79dBm */
	delay_us(100);
    RFM12_SetBaud(9600);             /* 9600 baud */
	delay_us(100);
    RFM12_SetPower(0, 6);             /* 1mW output powr, 120kHz frequency shift */

    /* show power up via LED on for 2 seconds */
    set_led(true);
    delay_ms(2000);
    set_led(false);

    #ifdef MODE_RECEIVER
    /* keep LED1 on to be able to distinguish between receive and transmitter */
    set_led(true);
    #endif
    

    /* main loop */
    while (1)
    {
        #ifdef MODE_TRANSMITTER
		delay_ms(1000); 
        transmit();
        
        set_led(true);
        delay_ms(50);
        set_led(false);
        delay_ms(50);    
        #endif

        #ifdef MODE_RECEIVER
        receive();
        
        set_led(false);
		delay_ms(500);
		set_led(true);
        #endif
    }
}

